# coding: utf-8

##
# Project: ScoPy
# Author: Marco Scarpetta <marcoscarpetta@mailoo.org>
# Copyright: 2012 Marco Scarpetta
# License: GPL-3+
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
# more details.
#
# On Debian GNU/Linux systems, the full text of the GNU General Public License
# can be found in the file /usr/share/common-licenses/GPL-3.
##

import sys, os
from gi.repository import Gtk,Clutter

APP_VERSION = '0.4.2 alpha'
APP_NAME = 'scopy'

#setting up gettext
import gettext
from gettext import gettext as _
gettext.bindtextdomain('scopy', '/usr/share/locale')
gettext.textdomain('scopy')

#recupero percorsi dove si trovano le immagini
percorso = sys.path[0][0:-4]
percorso_tap = percorso+'/data/images/tappeti/'
percorso_carte = percorso+'/data/images/carte/'
percorso_gui = percorso+'/data/ui/'
percorso_doc = percorso+'/doc/'

#funzione per spostamenti
def go_to(actor,x,y,time):
	an = actor.animatev(Clutter.AnimationMode.EASE_IN_OUT_QUAD, time,['x','y'],[x,y])
	return an

#destroys the given widget
def destroy(widget, response):
	widget.destroy()

#nomi dei file delle immagini delle carte
immagini = [
	["0.png", "1d.png", "2d.png", "3d.png", "4d.png", "5d.png", "6d.png", "7d.png", "8d.png", "9d.png", "10d.png"],
	["bg.png", "1c.png", "2c.png", "3c.png", "4c.png", "5c.png", "6c.png", "7c.png", "8c.png", "9c.png", "10c.png"],
	["mazzo.png", "1b.png", "2b.png", "3b.png", "4b.png", "5b.png", "6b.png", "7b.png", "8b.png", "9b.png", "10b.png"],
	["0.png", "1s.png", "2s.png", "3s.png", "4s.png", "5s.png", "6s.png", "7s.png", "8s.png", "9s.png", "10s.png"]
]

#varianti
varianti = [_('Classic scopa'), _('Cirulla'), _('Cucita'), _('Re Bello')]

#tempo in ms da aspettare dopo che una carta viene giocata, si regola con self.velocita
times=[1,3000,2000,1000]

#caricamento tipi di carte
tipi_di_carte = []
for cartella in os.listdir(percorso_carte):
	if cartella[0] != '.':
		tipi_di_carte.append(cartella)
tipi_di_carte.sort()

#caricamento sfondi
sfondi = []
for cartella in os.listdir(percorso_tap):
	if cartella[0] != '.':
		sfondi.append(cartella[0:-4])
sfondi.sort()

class Obj():
	def __init__(self, stage, carta, opzioni, scoperta):
		self.uid = carta.uid
		if scoperta:
			self.object = Clutter.Texture.new_from_file(percorso_carte+opzioni['carte']+'/'+immagini[carta.palo][carta.valore])
			stage.add_actor(self.object)
		else:
			self.object = Clutter.Texture.new_from_file(percorso_carte+opzioni['carte']+'/'+immagini[1][0])
			stage.add_actor(self.object)

class Carte():
	def __init__(self):
		self.carte = []
	def __getitem__(self, uid):
		if type(uid) == int:
			for carta in self.carte:
				if carta.uid == uid:
					return carta.object
		else:
			for carta in self.carte:
				if carta.object == uid:
					return carta.uid
	def __delitem__(self, uid):
		n=0
		while n < len(self.carte):
				if self.carte[n].uid == uid:
					del self.carte[n]
				n=n+1
	def append(self, obj):
		self.carte.append(obj)